/*
 * accountdlg.cpp - dialogs for manipulating PsiAccounts
 * Copyright (C) 2001, 2002  Justin Karneges
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include"accountdlg.h"

#include<qpushbutton.h>
#include<qlistview.h>
#include<qlineedit.h>
#include<qcheckbox.h>
#include<qlabel.h>
#include<qlayout.h>
#include<qgroupbox.h>
#include<qcombobox.h>
#include<qradiobutton.h>
#include<qbuttongroup.h>
#include<qtabwidget.h>
#include<qwhatsthis.h>
#include"psicon.h"
#include"psiaccount.h"
#include"common.h"
#include"busywidget.h"
#include"kmp/client.h"
#include"kmp/stream.h"
#include"kmp/tasks.h"

using namespace Jabber;

//----------------------------------------------------------------------------
// AccountManageDlg
//----------------------------------------------------------------------------
class AccountManageItem : public QListViewItem
{
public:
	AccountManageItem(QListView *par, PsiAccount *_pa)
	:QListViewItem(par)
	{
		pa = _pa;
		updateInfo();
	}

	void updateInfo()
	{
		const UserAccount &acc = pa->userAccount();
		setText(0, pa->name());
		setPixmap(0, is.account);
		setText(1, acc.opt_vhost ? acc.vhost : acc.host);
		setText(2, pa->isActive() ? AccountManageDlg::tr("Active") : AccountManageDlg::tr("Not active"));
	}

	int rtti() const
	{
		return 8109;
	}

	PsiAccount *pa;
};

AccountManageDlg::AccountManageDlg(PsiCon *_psi, const char *name)
:AccountManageUI(0, name, false, WDestructiveClose)
{
	psi = _psi;
	psi->dialogRegister(this);

	setCaption(CAP(tr("Jabber Accounts")));

	// setup signals
	connect(pb_add, SIGNAL(clicked()), SLOT(add()));
	connect(pb_modify, SIGNAL(clicked()), SLOT(modify()));
	connect(pb_remove, SIGNAL(clicked()), SLOT(remove()));
	connect(pb_close, SIGNAL(clicked()), SLOT(close()));

	connect(lv_accs, SIGNAL(doubleClicked(QListViewItem *)), SLOT(modify(QListViewItem *)));
	connect(lv_accs, SIGNAL(selectionChanged(QListViewItem *)), SLOT(qlv_selectionChanged(QListViewItem *)));
	connect(psi, SIGNAL(accountAdded(PsiAccount *)), SLOT(accountAdded(PsiAccount *)));
	connect(psi, SIGNAL(accountUpdated(PsiAccount *)), SLOT(accountUpdated(PsiAccount *)));
	connect(psi, SIGNAL(accountRemoved(PsiAccount *)), SLOT(accountRemoved(PsiAccount *)));

	lv_accs->setAllColumnsShowFocus(true);
	lv_accs->setResizeMode(QListView::LastColumn);

	PsiAccountListIt it(psi->accountList());
	for(PsiAccount *pa; (pa = it.current()); ++it) {
		new AccountManageItem(lv_accs, pa);
	}

	if(lv_accs->childCount() > 0)
		lv_accs->setSelected(lv_accs->firstChild(), true);
	else
		qlv_selectionChanged(0);
}

AccountManageDlg::~AccountManageDlg()
{
	psi->dialogUnregister(this);
}

void AccountManageDlg::qlv_selectionChanged(QListViewItem *lvi)
{
	AccountManageItem *i = (AccountManageItem *)lvi;
	bool ok = i ? true: false;

	pb_modify->setEnabled(ok);
	pb_remove->setEnabled(ok);
}

void AccountManageDlg::add()
{
	AccountAddDlg *w = new AccountAddDlg(psi, 0);
	w->show();
}

void AccountManageDlg::modify()
{
	modify(lv_accs->currentItem());
}

void AccountManageDlg::modify(QListViewItem *lvi)
{
	AccountManageItem *i = (AccountManageItem *)lvi;
	if(!i)
		return;

	psi->modifyAccount(i->pa);
}

void AccountManageDlg::remove()
{
	AccountManageItem *i = (AccountManageItem *)lv_accs->currentItem();
	if(!i)
		return;

	if(i->pa->isActive()) {
		QMessageBox::information(this, tr("Error"), tr("Unable to remove the account, as it is currently active."));
		return;
	}

	AccountRemoveDlg *w = new AccountRemoveDlg(i->pa->userAccount());
	int n = w->exec();
	if(n != QDialog::Accepted) {
		delete w;
		return;
	}
	delete w;
	psi->removeAccount(i->pa);
}

void AccountManageDlg::accountAdded(PsiAccount *pa)
{
	new AccountManageItem(lv_accs, pa);
}

void AccountManageDlg::accountUpdated(PsiAccount *pa)
{
	AccountManageItem *i;
	QListViewItemIterator it(lv_accs);
	for(; (i = (AccountManageItem *)it.current()) ; ++it) {
		if(i->pa == pa)
			break;
	}
	if(!i)
		return;

	i->updateInfo();
}

void AccountManageDlg::accountRemoved(PsiAccount *pa)
{
	AccountManageItem *i;
	QListViewItemIterator it(lv_accs);
	for(; (i = (AccountManageItem *)it.current()) ; ++it) {
		if(i->pa == pa)
			break;
	}
	if(!i)
		return;

	delete i;

	qlv_selectionChanged(lv_accs->currentItem());
}


//----------------------------------------------------------------------------
// AccountAddDlg
//----------------------------------------------------------------------------
AccountAddDlg::AccountAddDlg(PsiCon *_psi, QWidget *parent, const char *name)
:AccountAddUI(parent, name, false, WDestructiveClose)
{
	psi = _psi;
	psi->dialogRegister(this);

	setCaption(tr("Add Account"));

	connect(pb_close, SIGNAL(clicked()), SLOT(close()));
	connect(pb_add, SIGNAL(clicked()), SLOT(add()));
	connect(le_name, SIGNAL(textChanged(const QString &)), SLOT(setAddButton(const QString &)));

	QWhatsThis::add(ck_reg,
		tr("Check this option if you don't yet have a Jabber account "
		"and you want to register one.  Note that this will only work "
		"on servers that allow anonymous registration."));

	QString def = tr("Default");
	QString aname = def;
	int n = 0;
	while(1) {
		bool taken = false;
		PsiAccountListIt it(psi->accountList());
		for(PsiAccount *pa; (pa = it.current()); ++it) {
			if(aname == pa->name()) {
				taken = true;
				break;
			}
		}

		if(!taken)
			break;
		aname = def + '_' + QString::number(++n);
	}

	le_name->setText(aname);
	le_name->setFocus();
}

AccountAddDlg::~AccountAddDlg()
{
	psi->dialogUnregister(this);
}

void AccountAddDlg::add()
{
	if(ck_reg->isChecked()) {
		AccountRegDlg *w = new AccountRegDlg(this);
		int n = w->exec();
		if(n != QDialog::Accepted) {
			delete w;
			return;
		}

		QString host = w->host;
		int port = w->port;
		bool ssl = w->ssl;
		QString user = w->user;
		QString pass = w->pass;
		delete w;

		psi->createAccount(le_name->text(), host, port, ssl, user, pass);
	}
	else {
		psi->createAccount(le_name->text());
	}

	close();
}

void AccountAddDlg::setAddButton(const QString &s)
{
	pb_add->setEnabled(!s.isEmpty());
}


//----------------------------------------------------------------------------
// AccountModifyDlg
//----------------------------------------------------------------------------
AccountModifyDlg::AccountModifyDlg(PsiAccount *_pa, bool _haveSSL, QWidget *parent, const char *name)
:AccountModifyUI(parent, name, false, WDestructiveClose)
{
	pa = _pa;
	pa->dialogRegister(this);

	setCaption(tr("Account Properties"));
	setIcon(is.account);

	resize(minimumSize());

	haveSSL = _haveSSL;
	const UserAccount &acc = pa->userAccount();

	connect(pb_close, SIGNAL(clicked()), SLOT(reject()));
	connect(ck_pass, SIGNAL(toggled(bool)), le_pass, SLOT(setEnabled(bool)));
	connect(ck_vhost, SIGNAL(toggled(bool)), SLOT(vhostToggled(bool)));
	connect(pb_save, SIGNAL(clicked()), SLOT(save()));

	connect(le_host, SIGNAL(textChanged(const QString &)), SLOT(updateVhost()));

	le_pass->setEnabled(false);
	le_vhost->setEnabled(false);

	cb_proxy_type->setCurrentItem(acc.proxy_type);
	le_proxy_host->setText(acc.proxy_host);
	le_proxy_port->setText(QString::number(acc.proxy_port));
	le_proxy_user->setText(acc.proxy_user);
	le_proxy_pass->setText(acc.proxy_pass);
	connect(cb_proxy_type, SIGNAL(activated(int)), SLOT(proxyEnabled(int)));
	proxyEnabled(acc.proxy_type);

	connect(pb_vcard, SIGNAL(clicked()), SLOT(detailsVCard()));
	connect(pb_changepw, SIGNAL(clicked()), SLOT(detailsChangePW()));

	le_name->setText(acc.name);
	le_host->setText(acc.host);
	le_port->setText(QString::number(acc.port));
	if(acc.opt_ssl && haveSSL)
		ck_ssl->setChecked(acc.opt_ssl);
	connect(ck_ssl, SIGNAL(toggled(bool)), SLOT(sslToggled(bool)));

	le_user->setText(acc.user);
	if(acc.opt_pass) {
		ck_pass->setChecked(true);
		le_pass->setText(acc.pass);
	}

	if(acc.opt_vhost) {
		ck_vhost->setChecked(true);
		le_vhost->setText(acc.vhost);
	}

	le_resource->setText(acc.resource);
	le_priority->setText(QString::number(acc.priority));

	ck_plain->setChecked(acc.opt_plain);
	ck_auto->setChecked(acc.opt_auto);
	ck_reconn->setChecked(acc.opt_reconn);
	ck_log->setChecked(acc.opt_log);
	ck_keepAlive->setChecked(acc.opt_keepAlive);
	ck_ignoreSSLWarnings->setChecked(acc.opt_ignoreSSLWarnings);

	updateVhost();

	if(le_name->text().isEmpty())
		le_name->setFocus();
	else
		pb_save->setFocus();

	// QWhatsThis helpers
	QWhatsThis::add(ck_plain,
		tr("Normally, Psi logs in using the <i>digest</i> authentication "
		"method.  Check this box to force a plain text login "
		"to the Jabber server. Use this option only if you have "
		"problems connecting with the normal login procedure, as it "
		"makes your connection potentially vulnerable to "
		"attacks."));
	QWhatsThis::add(ck_auto,
		tr("Automatically login to this account on Psi startup.  Useful if "
		"you have Psi automatically launched when an Internet "
		"connection is detected."));
	QWhatsThis::add(ck_reconn,
		tr("Makes Psi try to reconnect if the connection was broken.  "
		"Useful, if you have an unstable connection and have to "
		"reconnect often."));
	QWhatsThis::add(ck_log,
		tr("Keep a log of message history.  Disable this "
		"option if you want to conserve disk space or if you need "
		"maximum security."));
	QWhatsThis::add(ck_keepAlive,
		tr("Sends so called \"Keep-alive\" packets periodically.  "
		"It is useful if your connection is set to be "
		"automatically disconnected after a certain period of "
		"inactivity (for example, by your ISP) and you want to keep it "
		"up all the time."));
	QWhatsThis::add(ck_ignoreSSLWarnings,
		tr("Ignores all the SSL warnings, for example, about "
		"incorrect certificates.  Useful if your server doesn't "
		"use a validated SSL certificate and you are "
		"annoyed with warnings."));
	QWhatsThis::add(cb_proxy_type,
		tr("If you require a proxy server to connect, select the type of proxy here.") + "  " +
		tr("Consult your network administrator if necessary."));
	QWhatsThis::add(le_proxy_host,
		tr("Enter the host name of your proxy server.") + "  " +
		tr("Consult your network administrator if necessary."));
	QWhatsThis::add(le_proxy_port,
		tr("Enter the port number of your proxy server.") + "  " +
		tr("Consult your network administrator if necessary."));
	QWhatsThis::add(le_proxy_user,
		tr("Enter your proxy server login (username) "
		"or leave this field blank if the proxy server does not require it.") + "  " +
		tr("Consult your network administrator if necessary."));
	QWhatsThis::add(le_proxy_pass,
		tr("Enter your proxy server password "
		"or leave this field blank if the proxy server does not require it.") + "  " +
		tr("Consult your network administrator if necessary."));
	QWhatsThis::add(ck_ssl,
		tr("Check this option to use an encrypted SSL connection to "
		"the Jabber server.  You may use this option if your "
		"server supports it and if you have the necessary QSSL "
		"plugin installed.  For more information, check the "
		"Psi homepage."));
	QWhatsThis::add(ck_pass,
		tr("Check this option if you want Psi to remember your Jabber "
		"account password. Don't use this feature if you want "
		"maximum security and don't want to be compromised even "
		"if someone would break in your system and steal your "
		"configuration files."));
	QWhatsThis::add(ck_vhost,
		tr("Use this option for manual configuration of your Jabber host "
		"if it is not the same as the host you are connecting to.  This option is mostly useful "
		"if you have some sort of proxy route on your "
		"local machine (i.e. you connect to localhost), but your "
		"account is registered on an external server."));
	QWhatsThis::add(le_resource,
		tr("You can have multiple clients connected to the Jabber server "
		"with your single account.  Each login is distinguished by a \"resource\" "
		"name, which you can specify in this field."));
	QWhatsThis::add(le_priority,
		tr("<p>You can have multiple clients connected to the Jabber "
		"server with your single account.  In such a situation, "
		"the client with the highest priority (that is specified in "
		"this field) will be the one that will receive "
		"all incoming events.</p>"
		"<p>For example, if you have a permanent connection "
		"to the Internet at your work location, and have a dial-up at home, "
		"you can have your Jabber client permanently running at work "
		"with a low priority, and you can still use the same account "
		"from home, using a client with higher priority to "
		"temporary \"disable\" the lower priority client at work.</p>"));
}

AccountModifyDlg::~AccountModifyDlg()
{
	pa->dialogUnregister(this);
}

void AccountModifyDlg::setPassword(const QString &pw)
{
	le_pass->setText(pw);
}

void AccountModifyDlg::sslToggled(bool on)
{
	if(on && !haveSSL) {
		QMessageBox::information(this, tr("SSL error"), tr("SSL not supported.\nReason:\n%1").arg(Stream::SSLUnsupportedReason()) );
		ck_ssl->setChecked(false);
		return;
	}

	le_port->setText(on ? "5223": "5222");
}

void AccountModifyDlg::vhostToggled(bool on)
{
	le_vhost->setEnabled(on);
	if(!on)
		updateVhost();
}

void AccountModifyDlg::updateVhost()
{
	if(!ck_vhost->isChecked()) {
		le_vhost->setText(le_host->text());
	}
}

void AccountModifyDlg::proxyEnabled(int proxytype)
{
	if(proxytype == 0) {
		gb_settings->setEnabled(false);
	}
	else {
		gb_settings->setEnabled(true);
		le_proxy_port->setText("8080");
	}
}

void AccountModifyDlg::detailsVCard()
{
	pa->changeVCard();
}

void AccountModifyDlg::detailsChangePW()
{
	pa->changePW();
}

void AccountModifyDlg::save()
{
	UserAccount acc = pa->userAccount();

	if(le_name->text().isEmpty()) {
		QMessageBox::information(this, tr("Error"), tr("You must specify a name for the account before you may save it."));
		return;
	}

	acc.name = le_name->text();
	acc.host = le_host->text();
	acc.port = le_port->text().toInt();
	acc.opt_ssl = ck_ssl->isChecked();
	acc.user = le_user->text();
	acc.opt_pass = ck_pass->isChecked();
	if(acc.opt_pass)
		acc.pass = le_pass->text();
	else
		acc.pass = "";
	acc.opt_vhost = ck_vhost->isChecked();
	acc.vhost = le_vhost->text();
	acc.resource = le_resource->text();
	acc.priority = le_priority->text().toInt();

	acc.opt_plain = ck_plain->isChecked();
	acc.opt_auto = ck_auto->isChecked();
	acc.opt_reconn = ck_reconn->isChecked();
	acc.opt_log = ck_log->isChecked();
	acc.opt_keepAlive = ck_keepAlive->isChecked();
	acc.opt_ignoreSSLWarnings = ck_ignoreSSLWarnings->isChecked();

	acc.proxy_type = cb_proxy_type->currentItem();
	acc.proxy_host = le_proxy_host->text();
	acc.proxy_port = le_proxy_port->text().toInt();
	acc.proxy_user = le_proxy_user->text();
	acc.proxy_pass = le_proxy_pass->text();

	pa->setUserAccount(acc);

	if(pa->isActive()) {
		QMessageBox::information(this, tr("Warning"), tr("This account is currently active, so certain changes may not take effect until the next login."));
	}

	accept();
}


//----------------------------------------------------------------------------
// AccountRegDlg
//----------------------------------------------------------------------------
AccountRegDlg::AccountRegDlg(QWidget *parent, const char *name)
:AccountRegUI(parent, name, false)
{
	setCaption(tr("Register Account"));

	busy = new BusyWidget(this);
	int n = baselayout->findWidget(lb_busywidget);
	baselayout->insertWidget(n+1, busy);
	delete lb_busywidget;

	connect(pb_close, SIGNAL(clicked()), SLOT(close()));
	connect(pb_reg, SIGNAL(clicked()), SLOT(reg()));
	connect(ck_ssl, SIGNAL(toggled(bool)), SLOT(sslToggled(bool)));

	connect(cb_proxy_type, SIGNAL(activated(int)), SLOT(proxyEnabled(int)));
	proxyEnabled(0);

	le_port->setText("5222");
	le_host->setFocus();

	client = new Client;
	connect(client, SIGNAL(handshaken()), SLOT(client_handshaken()));
	connect(client, SIGNAL(error(const StreamError &)), SLOT(client_error(const StreamError &)));
	connect(client, SIGNAL(debugText(const QString &)), SLOT(client_debugText(const QString &)));

	haveSSL = Stream::isSSLSupported();
}

AccountRegDlg::~AccountRegDlg()
{
	delete client;
}

void AccountRegDlg::closeEvent(QCloseEvent *e)
{
	e->ignore();
	reject();
}

void AccountRegDlg::done(int r)
{
	if(busy->isActive()) {
		int n = QMessageBox::information(this, tr("Warning"), tr("Are you sure you want to cancel the registration?"), tr("&Yes"), tr("&No"));
		if(n != 0)
			return;
	}
	QDialog::done(r);
}

void AccountRegDlg::proxyEnabled(int proxytype)
{
	if(proxytype == 0)
		gb_settings->setEnabled(false);
	else
		gb_settings->setEnabled(true);
}

void AccountRegDlg::sslToggled(bool on)
{
	if(on && !haveSSL) {
		QMessageBox::information(this, tr("SSL error"), tr("SSL not supported.\nReason:\n%1").arg(Stream::SSLUnsupportedReason()) );
		ck_ssl->setChecked(false);
		return;
	}

	le_port->setText(on ? "5223": "5222");
}

void AccountRegDlg::reg()
{
	// sanity check
	if(le_user->text().isEmpty() || le_host->text().isEmpty() || le_pass->text().isEmpty()) {
		QMessageBox::information(this, tr("Error"), tr("You must fill out the fields properly before you can register."));
		return;
	}

	if(le_pass->text() != le_confirm->text()) {
		QMessageBox::information(this, tr("Error"), tr("Password and confirmation do not match.  Please enter them again."));
		le_pass->setText("");
		le_confirm->setText("");
		le_pass->setFocus();
		return;
	}

	busy->start();
	block();

	ssl = ck_ssl->isChecked();
	host = le_host->text();
	port = le_port->text().toInt();
	user = le_user->text();
	pass = le_pass->text();

	int proxy_type = cb_proxy_type->currentItem();
	QString proxy_host = le_proxy_host->text();
	int proxy_port = le_proxy_port->text().toInt();
	QString proxy_user = le_proxy_user->text();
	QString proxy_pass = le_proxy_pass->text();

	StreamProxy p;
	p.setType(proxy_type);
	p.setHost(proxy_host);
	p.setPort(proxy_port);
	if(!proxy_user.isEmpty()) {
		p.setUseAuth(true);
		p.setUser(proxy_user);
		p.setPass(proxy_pass);
	}
	client->setProxy(p);
	client->setSSLEnabled(ssl);
	client->connectToHost(host, port);
}

void AccountRegDlg::client_handshaken()
{
	// try to register an account
	JT_Register *reg = new JT_Register(client->rootTask());
	connect(reg, SIGNAL(finished()), SLOT(reg_finished()));
	reg->reg(user, pass);
	reg->go(true);
}

void AccountRegDlg::client_error(const StreamError &err)
{
	if(err.isWarning()) {
		closeDialogs(this);
		int n = QMessageBox::warning(0, CAP(tr("Warning")), err.toString(), tr("Ignore"), tr("Disconnect"));
		if(n == 0) {
			client->continueAfterWarning();
		}
		else {
			client->close();
			busy->stop();
			unblock();
		}
		return;
	}

	client->close();
	busy->stop();
	unblock();
	closeDialogs(this);
	QMessageBox::critical(this, tr("Error"), QString(tr("There was an error communicating with the Jabber server:\n%1")).arg(err.toString()));
}

void AccountRegDlg::reg_finished()
{
	JT_Register *reg = (JT_Register *)sender();

	client->close();
	busy->stop();

	if(reg->success()) {
		closeDialogs(this);
		QMessageBox::information(this, tr("Success"), tr("The account was registered successfully."));
		accept();
		return;
	}

	unblock();
	closeDialogs(this);
	QMessageBox::critical(this, tr("Error"), QString(tr("There was an error registering the account.\nReason: %1")).arg(reg->statusString()));
}

void AccountRegDlg::client_debugText(const QString &)
{
	//printf("%s", str.latin1());
	//fflush(stdout);
}

void AccountRegDlg::block()
{
	tab_account->setEnabled(false);
	pb_reg->setEnabled(false);
}

void AccountRegDlg::unblock()
{
	tab_account->setEnabled(true);
	pb_reg->setEnabled(true);
	le_user->setFocus();
}


//----------------------------------------------------------------------------
// AccountRemoveDlg
//----------------------------------------------------------------------------
class AccountRemoveDlg::Private
{
public:
	Private() {}

	UserAccount acc;
	QButtonGroup *bg;
};

AccountRemoveDlg::AccountRemoveDlg(const UserAccount &acc, QWidget *parent, const char *name)
:AccountRemoveUI(parent, name, false)
{
	d = new Private;
	d->acc = acc;

	setCaption(tr("Remove Account"));

	busy = new BusyWidget(this);
	int n = baselayout->findWidget(lb_busywidget);
	baselayout->insertWidget(n+1, busy);
	delete lb_busywidget;

	connect(pb_close, SIGNAL(clicked()), SLOT(close()));
	connect(pb_remove, SIGNAL(clicked()), SLOT(remove()));

	d->bg = new QButtonGroup(0);
	d->bg->insert(rb_remove, 0);
	d->bg->insert(rb_removeAndUnreg, 1);
	connect(d->bg, SIGNAL(clicked(int)), SLOT(bg_clicked(int)));
	d->bg->setButton(0);
	bg_clicked(0);

	pb_close->setFocus();

	client = new Client;
	connect(client, SIGNAL(handshaken()), SLOT(client_handshaken()));
	connect(client, SIGNAL(error(const StreamError &)), SLOT(client_error(const StreamError &)));
	connect(client, SIGNAL(authFinished(bool, int, const QString &)), SLOT(client_authFinished(bool, int, const QString &)));
	connect(client, SIGNAL(debugText(const QString &)), SLOT(client_debugText(const QString &)));

	haveSSL = Stream::isSSLSupported();
}

AccountRemoveDlg::~AccountRemoveDlg()
{
	delete client;

	delete d->bg;
	delete d;
}

void AccountRemoveDlg::closeEvent(QCloseEvent *e)
{
	e->ignore();
	reject();
}

void AccountRemoveDlg::done(int r)
{
	if(busy->isActive()) {
		int n = QMessageBox::information(this, tr("Warning"), tr("Are you sure you want to cancel the unregistration?"), tr("&Yes"), tr("&No"));
		if(n != 0)
			return;
	}
	QDialog::done(r);
}

void AccountRemoveDlg::bg_clicked(int x)
{
	if(x == 0) {
		lb_pass->setEnabled(false);
		le_pass->setEnabled(false);
	}
	else if(x == 1) {
		lb_pass->setEnabled(true);
		le_pass->setEnabled(true);
		le_pass->setFocus();
	}
}

void AccountRemoveDlg::remove()
{
	bool unreg = rb_removeAndUnreg->isChecked();

	if(unreg) {
		if(le_pass->text() != d->acc.pass) {
			QMessageBox::information(this, tr("Error"), tr("Password does not match account.  Please try again."));
			le_pass->setFocus();
			return;
		}
	}

	int n = QMessageBox::information(this, tr("Warning"), tr("Are you sure you want to remove <b>%1</b> ?").arg(d->acc.name), tr("&Yes"), tr("&No"));
	if(n != 0)
		return;

	if(!unreg) {
		accept();
		return;
	}

	busy->start();
	gb_account->setEnabled(false);
	pb_remove->setEnabled(false);

	StreamProxy p;
	p.setType(d->acc.proxy_type);
	p.setHost(d->acc.proxy_host);
	p.setPort(d->acc.proxy_port);
	if(!d->acc.proxy_user.isEmpty()) {
		p.setUseAuth(true);
		p.setUser(d->acc.proxy_user);
		p.setPass(d->acc.proxy_pass);
	}
	client->setProxy(p);
	client->setSSLEnabled(d->acc.opt_ssl);
	client->connectToHost(d->acc.host, d->acc.port);
}

void AccountRemoveDlg::client_handshaken()
{
	// authorize
	if(d->acc.opt_plain)
		client->authPlain(d->acc.user, d->acc.pass, d->acc.resource);
	else
		client->authDigest(d->acc.user, d->acc.pass, d->acc.resource);
}

void AccountRemoveDlg::client_error(const StreamError &err)
{
	if(err.isWarning()) {
		closeDialogs(this);
		int n = QMessageBox::warning(0, CAP(tr("Warning")), err.toString(), tr("Ignore"), tr("Disconnect"));
		if(n == 0) {
			client->continueAfterWarning();
		}
		else {
			client->close();
			busy->stop();
			gb_account->setEnabled(true);
			pb_remove->setEnabled(true);
		}
		return;
	}

	client->close();
	busy->stop();
	gb_account->setEnabled(true);
	pb_remove->setEnabled(true);

	closeDialogs(this);
	QMessageBox::critical(this, tr("Error"), QString(tr("There was an error communicating with the Jabber server:\n%1")).arg(err.toString()));
}

void AccountRemoveDlg::client_authFinished(bool success, int, const QString &str)
{
	if(success) {
		// try to unregister an account
		JT_Register *reg = new JT_Register(client->rootTask());
		connect(reg, SIGNAL(finished()), SLOT(unreg_finished()));
		reg->unreg();
		reg->go(true);
	}
	else {
		client->close();
		busy->stop();
		gb_account->setEnabled(true);
		pb_remove->setEnabled(true);
		closeDialogs(this);
		QMessageBox::critical(0, CAP(tr("Error")), tr("There was an error authenticating with the Jabber server.\nDetails: %1").arg(str));
	}
}

void AccountRemoveDlg::unreg_finished()
{
	JT_Register *reg = (JT_Register *)sender();

	client->close();
	busy->stop();

	if(reg->success()) {
		closeDialogs(this);
		QMessageBox::information(this, tr("Success"), tr("The account was unregistered successfully."));
		accept();
		return;
	}

	gb_account->setEnabled(true);
	pb_remove->setEnabled(true);
	closeDialogs(this);
	QMessageBox::critical(this, tr("Error"), QString(tr("There was an error unregistering the account.\nReason: %1")).arg(reg->statusString()));
}

void AccountRemoveDlg::client_debugText(const QString &)
{
	//printf("%s", str.latin1());
	//fflush(stdout);
}
