/*
 * infodlg.cpp - handle vcard
 * Copyright (C) 2001, 2002  Justin Karneges
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include"infodlg.h"

#include<qlayout.h>
#include<qlabel.h>
#include<qtabwidget.h>
#include<qpushbutton.h>
#include<qlineedit.h>
#include<qmessagebox.h>
#include<qtextedit.h>
#include"kmp/jid.h"
#include"kmp/vcard.h"
#include"kmp/tasks.h"
#include"psiaccount.h"
#include"busywidget.h"
#include"iconset.h"
#include"common.h"


using namespace Jabber;

class InfoDlg::Private
{
public:
	Private() {}

	int type;
	Jid jid;
	VCard vcard;
	PsiAccount *pa;
	BusyWidget *busy;
	bool te_edited;
	int actionType;
	JT_VCard *jt;
};

InfoDlg::InfoDlg(int type, const Jid &j, const VCard &vcard, PsiAccount *pa, QWidget *parent, const char *name)
:InfoUI(parent, name, false, WDestructiveClose)
{
	d = new Private;
	d->type = type;
	d->jid = j;
	d->vcard = vcard;
	d->pa = pa;
	d->te_edited = false;
	d->jt = 0;
	d->pa->dialogRegister(this, j);

	d->busy = new BusyWidget(this);
	replaceWidget(lb_busywidget, d->busy);

	setCaption(d->jid.full());
	setIcon(is.info);

	connect(pb_refresh, SIGNAL(clicked()), this, SLOT(doRefresh()));
	connect(te_desc, SIGNAL(textChanged()), this, SLOT(textChanged()));

	if(d->type == Self) {
		connect(pb_submit, SIGNAL(clicked()), this, SLOT(doSubmit()));
	}
	else {
		pb_submit->hide();
		setReadOnly(true);
	}

	setData(d->vcard);
}

InfoDlg::~InfoDlg()
{
	d->pa->dialogUnregister(this);
	delete d;
}

void InfoDlg::closeEvent(QCloseEvent *e)
{
	e->ignore();
	reject();
}

void InfoDlg::done(int r)
{
	// don't close if submitting
	if(d->busy->isActive() && d->actionType == 1)
		return;

	if(d->type == Self && edited()) {
		int n = QMessageBox::information(this, tr("Warning"), tr("You have not published your account information changes.\nAre you sure you want to discard them?"), tr("Close and discard"), tr("Don't close"));
		if(n != 0)
			return;
	}

	// cancel active transaction (refresh only)
	if(d->busy->isActive() && d->actionType == 0) {
		delete d->jt;
		d->jt = 0;
	}

	QDialog::done(r);
}

void InfoDlg::jt_finished()
{
	d->busy->stop();
	pb_refresh->setEnabled(true);
	pb_submit->setEnabled(true);
	pb_close->setEnabled(true);
	fieldsEnable(true);

	if(d->jt->success()) {
		if(d->actionType == 0) {
			d->vcard = d->jt->vcard();
			setData(d->vcard);
		}
		else if(d->actionType == 1) {
			d->vcard = d->jt->vcard();
			setData(d->vcard);
			d->pa->setNick(d->vcard.field[vNickname]);
			QMessageBox::information(this, tr("Success"), tr("Your account information has been published."));
		}
	}
	else {
		if(d->actionType == 0) {
			if(d->type == Self)
				QMessageBox::critical(this, tr("Error"), tr("Unable to retrieve your account information.  Perhaps you haven't entered any yet."));
			else
				QMessageBox::critical(this, tr("Error"), tr("Unable to retrieve information about this contact.\nReason: %1").arg(d->jt->statusString()));
		}
		else {
			QMessageBox::critical(this, tr("Error"), tr("Unable to publish your account information.\nReason: %1").arg(d->jt->statusString()));
		}
	}

	d->jt = 0;
}

void InfoDlg::setData(const VCard &i)
{
	le_fullname->setText(i.field[vFullname]);
	le_nickname->setText(i.field[vNickname]);
	le_bday->setText(i.field[vBday]);
	le_email->setText(i.field[vEmail]);
	le_homepage->setText(i.field[vHomepage]);
	le_phone->setText(i.field[vPhone]);
	le_street->setText(i.field[vStreet]);
	le_ext->setText(i.field[vExt]);
	le_city->setText(i.field[vCity]);
	le_state->setText(i.field[vState]);
	le_pcode->setText(i.field[vPcode]);
	le_country->setText(i.field[vCountry]);
	le_orgName->setText(i.field[vOrgName]);
	le_orgUnit->setText(i.field[vOrgUnit]);
	le_title->setText(i.field[vTitle]);
	le_role->setText(i.field[vRole]);
	te_desc->setText(i.field[vDesc]);

	setEdited(false);
}

void InfoDlg::fieldsEnable(bool x)
{
	le_fullname->setEnabled(x);
	le_nickname->setEnabled(x);
	le_bday->setEnabled(x);
	le_email->setEnabled(x);
	le_homepage->setEnabled(x);
	le_phone->setEnabled(x);

	le_street->setEnabled(x);
	le_ext->setEnabled(x);
	le_city->setEnabled(x);
	le_state->setEnabled(x);
	le_pcode->setEnabled(x);
	le_country->setEnabled(x);

	le_orgName->setEnabled(x);
	le_orgUnit->setEnabled(x);
	le_title->setEnabled(x);
	le_role->setEnabled(x);
	te_desc->setEnabled(x);

	setEdited(false);
}

void InfoDlg::setEdited(bool x)
{
	le_fullname->setEdited(x);
	le_nickname->setEdited(x);
	le_bday->setEdited(x);
	le_email->setEdited(x);
	le_homepage->setEdited(x);
	le_phone->setEdited(x);
	le_street->setEdited(x);
	le_ext->setEdited(x);
	le_city->setEdited(x);
	le_state->setEdited(x);
	le_pcode->setEdited(x);
	le_country->setEdited(x);
	le_orgName->setEdited(x);
	le_orgUnit->setEdited(x);
	le_title->setEdited(x);
	le_role->setEdited(x);

	d->te_edited = x;
}

bool InfoDlg::edited()
{
	bool x = false;

	if(le_fullname->edited()) x = true;
	if(le_nickname->edited()) x = true;
	if(le_bday->edited()) x = true;
	if(le_email->edited()) x = true;
	if(le_homepage->edited()) x = true;
	if(le_phone->edited()) x = true;
	if(le_street->edited()) x = true;
	if(le_ext->edited()) x = true;
	if(le_city->edited()) x = true;
	if(le_state->edited()) x = true;
	if(le_pcode->edited()) x = true;
	if(le_country->edited()) x = true;
	if(le_orgName->edited()) x = true;
	if(le_orgUnit->edited()) x = true;
	if(le_title->edited()) x = true;
	if(le_role->edited()) x = true;
	if(d->te_edited) x = true;

	return x;
}

void InfoDlg::setReadOnly(bool x)
{
	le_fullname->setReadOnly(x);
	le_nickname->setReadOnly(x);
	le_bday->setReadOnly(x);
	le_email->setReadOnly(x);
	le_homepage->setReadOnly(x);
	le_phone->setReadOnly(x);
	le_street->setReadOnly(x);
	le_ext->setReadOnly(x);
	le_city->setReadOnly(x);
	le_state->setReadOnly(x);
	le_pcode->setReadOnly(x);
	le_country->setReadOnly(x);
	le_orgName->setReadOnly(x);
	le_orgUnit->setReadOnly(x);
	le_title->setReadOnly(x);
	le_role->setReadOnly(x);
	te_desc->setReadOnly(x);
}

void InfoDlg::doRefresh()
{
	if(!d->pa->checkConnected(this))
		return;
	if(!pb_refresh->isEnabled())
		return;
	if(d->busy->isActive())
		return;

	pb_submit->setEnabled(false);
	pb_refresh->setEnabled(false);
	fieldsEnable(false);

	d->actionType = 0;
	d->busy->start();

	d->jt = new JT_VCard(d->pa->client()->rootTask());
	connect(d->jt, SIGNAL(finished()), SLOT(jt_finished()));
	d->jt->get(d->jid);
	d->jt->go(true);
}

void InfoDlg::doSubmit()
{
	if(!d->pa->checkConnected(this))
		return;
	if(!pb_submit->isEnabled())
		return;
	if(d->busy->isActive())
		return;

	VCard submit_vcard = makeVCard();

	pb_submit->setEnabled(false);
	pb_refresh->setEnabled(false);
	pb_close->setEnabled(false);
	fieldsEnable(false);

	d->actionType = 1;
	d->busy->start();

	d->jt = new JT_VCard(d->pa->client()->rootTask());
	connect(d->jt, SIGNAL(finished()), SLOT(jt_finished()));
	d->jt->set(submit_vcard);
	d->jt->go(true);
}

VCard InfoDlg::makeVCard()
{
	VCard v;

	v.field[vFullname] = le_fullname->text();
	v.field[vNickname] = le_nickname->text();
	v.field[vBday] = le_bday->text();
	v.field[vEmail] = le_email->text();
	v.field[vHomepage] = le_homepage->text();
	v.field[vPhone] = le_phone->text();

	v.field[vStreet] = le_street->text();
	v.field[vExt] = le_ext->text();
	v.field[vCity] = le_city->text();
	v.field[vState] = le_state->text();
	v.field[vPcode] = le_pcode->text();
	v.field[vCountry] = le_country->text();

	v.field[vOrgName] = le_orgName->text();
	v.field[vOrgUnit] = le_orgUnit->text();
	v.field[vTitle] = le_title->text();
	v.field[vRole] = le_role->text();
	v.field[vDesc] = te_desc->text();

	return v;
}

void InfoDlg::textChanged()
{
	d->te_edited = true;
}
