/*
 * dtcp.cpp - direct connection protocol via tcp
 * Copyright (C) 2001, 2002  Justin Karneges
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include"dtcp.h"

#include<qcstring.h>
#include<qsocket.h>
#include<qptrlist.h>
#include<qtimer.h>
#include"ndns.h"
#include"xmlcommon.h"

#include<stdlib.h>

using namespace Jabber;

static int num_dsh = 0;
static int num_conn = 0;

//----------------------------------------------------------------------------
// DTCPSocketHandler
//----------------------------------------------------------------------------
class DTCPSocketHandler::Private
{
public:
	Private() {}

	DTCPManager *m;
	int mode;
	QSocket *sock;
	Jid peer;
	QString key, verify;
	NDns ndns;
	QString host;
	int port;
	bool established;
	QByteArray recvbuf, hbuf;
	bool waitingOn;
};

DTCPSocketHandler::DTCPSocketHandler(DTCPManager *m)
:QObject(0)
{
	++num_dsh;
	printf("+++ [%d] DTCPSocketHandler\n", num_dsh);
	d = new Private;
	d->m = m;
	d->mode = Active;
	d->established = false;
	d->waitingOn = false;

	connect(&d->ndns, SIGNAL(resultsReady()), SLOT(ndns_done()));

	d->sock = new QSocket;
	connect(d->sock, SIGNAL(connected()), SLOT(sock_connected()));
	connect(d->sock, SIGNAL(connectionClosed()), SLOT(sock_connectionClosed()));
	connect(d->sock, SIGNAL(delayedCloseFinished()), SLOT(sock_delayedCloseFinished()));
	connect(d->sock, SIGNAL(readyRead()), SLOT(sock_readyRead()));
	connect(d->sock, SIGNAL(bytesWritten(int)), SLOT(sock_bytesWritten(int)));
	connect(d->sock, SIGNAL(error(int)), SLOT(sock_error(int)));
}

DTCPSocketHandler::~DTCPSocketHandler()
{
	--num_dsh;
	printf("--- [%d] DTCPSocketHandler\n", num_dsh);
	close();
	delete d->sock;
	delete d;
}

void DTCPSocketHandler::handle(const QString &host, int port, const Jid &peer, const QString &key, const QString &verify)
{
	d->mode = Active;
	d->host = host;
	d->port = port;
	d->peer = peer;
	d->key = key;
	d->verify = verify;
	d->recvbuf.resize(0);
	d->hbuf.resize(0);
	d->established = false;

	d->ndns.resolve(d->host.latin1());
}

void DTCPSocketHandler::handle(int s)
{
	d->mode = Passive;
	d->established = false;
	d->waitingOn = false;
	d->sock->setSocket(s);

	if(d->sock->bytesAvailable() > 0)
		sock_readyRead();
}

void DTCPSocketHandler::close()
{
	d->sock->close();
}

bool DTCPSocketHandler::waitingOn() const
{
	return d->waitingOn;
}

int DTCPSocketHandler::mode() const
{
	return d->mode;
}

Jid DTCPSocketHandler::peer() const
{
	return d->peer;
}

QString DTCPSocketHandler::key() const
{
	return d->key;
}

QString DTCPSocketHandler::verify() const
{
	return d->verify;
}

QHostAddress DTCPSocketHandler::peerAddress() const
{
	return d->sock->peerAddress();
}

bool DTCPSocketHandler::isEstablished() const
{
	return d->established;
}

void DTCPSocketHandler::write(const QByteArray &buf)
{
	if(d->established)
		d->sock->writeBlock(buf.data(), buf.size());
}

QByteArray DTCPSocketHandler::read()
{
	QByteArray a;

	if(canRead()) {
		a = d->recvbuf;
		a.detach();
		d->recvbuf.resize(0);
	}

	return a;
}

bool DTCPSocketHandler::canRead() const
{
	return (d->recvbuf.size() > 0 ? true: false);
}

void DTCPSocketHandler::ndns_done()
{
	if(d->ndns.result() == 0) {
		doError("Unable to resolve hostname");
		return;
	}

	d->sock->connectToHost(d->ndns.resultString(), d->port);
}

void DTCPSocketHandler::sock_connected()
{
	printf("dsh: connected, sending key line\n");
	QString s = QString("key:") + d->key + '\r';
	QCString cs = s.utf8();
	d->sock->writeBlock(cs.data(), cs.length());
}

void DTCPSocketHandler::sock_connectionClosed()
{
	if(d->established) {
		connectionClosed();
	}
	else {
		doError("Disconnected");
	}
}

void DTCPSocketHandler::sock_delayedCloseFinished()
{
	if(d->established) {
		d->established = false;
		delayedCloseFinished();
	}
}

void DTCPSocketHandler::sock_readyRead()
{
	QByteArray block;
	int len = d->sock->bytesAvailable();
	block.resize(len);
	if(len < 1)
		len = 1;
	int actual = d->sock->readBlock(block.data(), len);
	if(actual < 1)
		return;
	block.resize(actual);

	printf("read %d bytes\n", block.size());

	if(d->established || d->waitingOn) {
		int oldsize = d->recvbuf.size();
		d->recvbuf.resize(oldsize + block.size());
		memcpy(d->recvbuf.data() + oldsize, block.data(), block.size());
		if(d->established)
			readyRead();
	}
	else {
		// read into hbuf
		int oldsize = d->hbuf.size();
		d->hbuf.resize(oldsize + block.size());
		memcpy(d->hbuf.data() + oldsize, block.data(), block.size());

		// process all 'lines'
		while(1) {
			bool found;
			QString line = extractLine(&d->hbuf, &found);
			if(!found)
				break;

			if(!processLine(line))
				return;
		}
	}
}

void DTCPSocketHandler::sock_bytesWritten(int x)
{
	if(d->established) {
		// echo
		bytesWritten(x);
	}
}

void DTCPSocketHandler::sock_error(int x)
{
	if(d->established) {
		d->established = false;
		// echo
		error(x);
	}
	else {
		doError("Socket error");
	}
}

void DTCPSocketHandler::doSuccess()
{
	d->established = true;
	result(true, "");
}

void DTCPSocketHandler::doError(const QString &str)
{
	result(false, str);
}

QString DTCPSocketHandler::extractLine(QByteArray *buf, bool *found)
{
	// scan for newline
	int n;
	for(n = 0; n < (int)buf->size(); ++n) {
		if(buf->at(n) == '\r') {
			QCString cstr;
			cstr.resize(n+1);
			memcpy(cstr.data(), buf->data(), n);
			++n; // hack off CR

			memmove(buf->data(), buf->data() + n, buf->size() - n);
			buf->resize(buf->size() - n);
			QString s = QString::fromUtf8(cstr);

			if(found)
				*found = true;
			return s;
		}
	}

	if(found)
		*found = false;
	return "";
}

bool DTCPSocketHandler::processLine(const QString &line)
{
	printf("processing line: [%s]\n", line.latin1());

	// active
	if(d->mode == Active) {
		QStringList l = QStringList::split(':', line);
		if(l.count() < 2 || l[0] != "ok") {
			d->sock->close();
			doError("Invalid header response received");
			return false;
		}
		QString verify = l[1];

		if(d->verify != verify) {
			d->sock->close();
			doError("Verification failed");
			return false;
		}
	}
	// passive
	else {
		QStringList l = QStringList::split(':', line);
		if(l.count() < 2 || l[0] != "key") {
			QCString cstr = "error:bad input\r";
			d->sock->writeBlock(cstr.data(), cstr.length());
			return true;
		}
		QString key = l[1];

		printf("searching for key: [%s]\n", key.latin1());
		DTCPConnection *c = d->m->findWaitingConnection(key);
		if(!c) {
			QCString cstr = "error:no such key\r";
			d->sock->writeBlock(cstr.data(), cstr.length());
			return true;
		}
		d->peer = c->peer();
		d->key = c->key();

		if(!c->hasVerify()) {
			printf("no verify yet.  let's wait for it.\n");
			d->waitingOn = true;
			return true;
		}

		d->verify = c->verify();

		QString str = QString("ok:") + d->verify + '\r';
		QCString cstr = str.utf8();
		d->sock->writeBlock(cstr.data(), cstr.length());
	}

	// on success
	doSuccess();
	if(d->hbuf.size() > 0) {
		d->recvbuf = d->hbuf;
		d->recvbuf.detach();
		d->hbuf.resize(0);
		readyRead();
	}

	return true;
}

void DTCPSocketHandler::tryVerify()
{
	if(d->waitingOn) {
		printf("trying to verify again...\n");
		DTCPConnection *c = d->m->findWaitingConnection(d->key);
		if(!c) {
			QCString cstr = "error:no such key\r";
			d->sock->writeBlock(cstr.data(), cstr.length());
			return;
		}
		if(!c->hasVerify()) {
			printf("no verify yet.  let's wait for it.\n");
			return;
		}

		d->verify = c->verify();
		d->waitingOn = false;

		QString str = QString("ok:") + d->verify + '\r';
		QCString cstr = str.utf8();
		d->sock->writeBlock(cstr.data(), cstr.length());

		doSuccess();
		if(d->hbuf.size() > 0) {
			d->recvbuf = d->hbuf;
			d->recvbuf.detach();
			d->hbuf.resize(0);
			readyRead();
		}
	}
}


//----------------------------------------------------------------------------
// DTCPConnection
//----------------------------------------------------------------------------
class DTCPConnection::Private
{
public:
	Private() {}

	DTCPManager *m;
	DTCPSocketHandler *sock;
	int state;
	Jid peer;
	QString key, verify;
	QDateTime ts;
	QStringList hostList;
	JT_DTCP *j;
	bool passive;
	QDomElement comment;
	QString iq_id;
	bool hasVerify;
	bool remote;
};

DTCPConnection::DTCPConnection(DTCPManager *m)
:JidLinkHandler(0)
{
	++num_conn;
	printf("+++ [%d] DTCPConnection\n", num_conn);
	d = new Private;
	d->m = m;
	d->sock = 0;
	d->j = 0;
	d->state = Idle;
	d->hasVerify = false;
	d->remote = false;
}

DTCPConnection::~DTCPConnection()
{
	--num_conn;
	printf("--- [%d] DTCPConnection\n", num_conn);
	close();
	delete d;
}

bool DTCPConnection::connectToJid(const Jid &peer, bool passive, const QDomElement &comment)
{
	if(d->state != Idle)
		return false;

	if(passive && !d->m->isActive())
		return false;

	d->state = Asking;
	d->peer = peer;
	d->key = d->m->genUniqueKey(peer);
	d->passive = passive;
	d->comment = comment;
	d->remote = false;

	d->m->link(this);

	QString dstr; dstr.sprintf("DTCPConnection: initiating request %s [%s], passive=%s\n", peer.full().latin1(), d->key.latin1(), passive ? "true": "false");
	d->m->client()->debug(dstr);

	d->j = new JT_DTCP(d->m->client()->rootTask());
	connect(d->j, SIGNAL(finished()), SLOT(dtcp_finished()));

	if(passive) {
		QStringList hl;
		QStringList hostList = d->m->hostList();
		for(QStringList::ConstIterator it = hostList.begin(); it != hostList.end(); ++it)
			hl += (*it) + ':' + QString::number(d->m->port());
		d->j->requestReverse(d->peer, d->key, hl, comment);
	}
	else {
		d->j->request(d->peer, d->key, comment);
	}

	d->j->go(true);

	return true;
}

void DTCPConnection::waitForSocket(const Jid &peer, const QString &key, const QString &verify, const QDomElement &comment, const QString &iq_id)
{
	d->state = WaitingForSocket;
	d->passive = false;
	d->peer = peer;
	d->key = key;
	d->verify = verify;
	d->comment = comment;
	d->iq_id = iq_id;
	d->ts = QDateTime::currentDateTime();

	d->hasVerify = true;
	d->remote = true;
	d->m->link(this);
}

void DTCPConnection::setSocketHandler(DTCPSocketHandler *s)
{
	d->sock = s;
	connect(d->sock, SIGNAL(connectionClosed()), SLOT(dsh_connectionClosed()));
	connect(d->sock, SIGNAL(delayedCloseFinished()), SLOT(dsh_delayedCloseFinished()));
	connect(d->sock, SIGNAL(readyRead()), SLOT(dsh_readyRead()));
	connect(d->sock, SIGNAL(bytesWritten(int)), SLOT(dsh_bytesWritten(int)));
	connect(d->sock, SIGNAL(error(int)), SLOT(dsh_error(int)));

	d->state = Active;
	d->peer = s->peer();
	d->key = s->key();

	QString dstr; dstr.sprintf("DTCPConnection: %s [%s] received successfully.\n", d->peer.full().latin1(), d->key.latin1());
	d->m->client()->debug(dstr);
	connected();
}

void DTCPConnection::waitForAccept(const Jid &peer, const QString &key, const QString &verify, const QStringList &hostList, const QDomElement &comment, const QString &iq_id)
{
	d->state = WaitingForAccept;
	d->passive = true;
	d->peer = peer;
	d->key = key;
	d->verify = verify;
	d->hostList = hostList;
	d->comment = comment;
	d->iq_id = iq_id;

	d->hasVerify = true;
	d->remote = true;
	d->m->link(this);
}

void DTCPConnection::conn()
{
	if(d->hostList.isEmpty()) {
		close();
		error(ErrEstablish);
		return;
	}

	QStringList hl = d->hostList;
	QString e = hl[0]; //[hl.count()-1];
	int n = e.find(':');
	QString host = e.mid(0, n);
	int port = e.mid(n+1).toInt();

	QString dstr; dstr.sprintf("DTCPConnection: connecting to %s:%d\n", host.latin1(), port);
	d->m->client()->debug(dstr);

	delete d->sock;
	d->sock = new DTCPSocketHandler(d->m);
	connect(d->sock, SIGNAL(result(bool, const QString &)), SLOT(dsh_result(bool, const QString &)));
	connect(d->sock, SIGNAL(connectionClosed()), SLOT(dsh_connectionClosed()));
	connect(d->sock, SIGNAL(delayedCloseFinished()), SLOT(dsh_delayedCloseFinished()));
	connect(d->sock, SIGNAL(readyRead()), SLOT(dsh_readyRead()));
	connect(d->sock, SIGNAL(bytesWritten(int)), SLOT(dsh_bytesWritten(int)));
	connect(d->sock, SIGNAL(error(int)), SLOT(dsh_error(int)));
	d->sock->handle(host, port, d->peer, d->key, d->verify);

	// cross-off this host
	QStringList::Iterator it = hl.begin(); //hl.end();
	//--it;
	hl.remove(it);
	d->hostList = hl;
}

void DTCPConnection::accept()
{
	if(d->state == WaitingForSocket) {
		d->state = Accepting;
		d->m->doAccept(this, d->iq_id);
	}
	else if(d->state == WaitingForAccept) {
		d->state = Connecting;
		d->m->doAcceptReverse(this, d->iq_id);
		conn();
	}
}

void DTCPConnection::close()
{
	if(d->state == Idle)
		return;

	if(d->state == Connecting) {
		QDomElement iq = createIQ(d->m->client()->doc(), "error", d->peer.full(), "");
		QDomElement query = d->m->client()->doc()->createElement("query");
		query.setAttribute("xmlns", "jabber:iq:dtcp");
		iq.appendChild(query);
		query.appendChild(textTag(d->m->client()->doc(), "key", d->key));
		QDomElement err = textTag(d->m->client()->doc(), "error", "I give up!");
		err.setAttribute("code", "500");
		iq.appendChild(err);
		d->m->client()->send(iq);
	}
	if(d->state == WaitingForSocket || d->state == WaitingForAccept) {
		d->m->doReject(this, d->iq_id);
	}

	reset();

	QString dstr; dstr.sprintf("DTCPConnection: closing %s [%s]\n", d->peer.full().latin1(), d->key.latin1());
	d->m->client()->debug(dstr);
}

bool DTCPConnection::isRemote() const
{
	return d->remote;
}

bool DTCPConnection::isPassive() const
{
	return d->passive;
}

int DTCPConnection::state() const
{
	return d->state;
}

Jid DTCPConnection::peer() const
{
	return d->peer;
}

QString DTCPConnection::key() const
{
	return d->key;
}

QString DTCPConnection::verify() const
{
	return d->verify;
}

QDomElement DTCPConnection::comment() const
{
	return d->comment;
}

bool DTCPConnection::isConnected() const
{
	if(d->state == Active)
		return true;
	else
		return false;
}

bool DTCPConnection::wantsIncomingSocket() const
{
	if((d->state == Accepting || (d->state == Asking && d->passive)) && !d->sock)
		return true;

	return false;
}

void DTCPConnection::write(const QByteArray &buf)
{
	if(d->sock && d->sock->isEstablished())
		d->sock->write(buf);
}

QByteArray DTCPConnection::read()
{
	if(d->sock && d->sock->isEstablished())
		return d->sock->read();
	else
		return QByteArray();
}

bool DTCPConnection::canRead() const
{
	if(d->sock && d->sock->isEstablished())
		return d->sock->canRead();
	else
		return false;
}

void DTCPConnection::dsh_result(bool success, const QString &str)
{
	if(success) {
		d->state = Active;

		QString dstr; dstr.sprintf("DTCPConnection: %s [%s] connected successfully.\n", d->peer.full().latin1(), d->key.latin1());
		d->m->client()->debug(dstr);
		connected();
	}
	else {
		printf("dtcpconnection::dsh_result: error: [%s]\n", str.latin1());
		if(d->state == Connecting) {
			conn();
		}
		else {
			reset();
			error(ErrEstablish);
		}
		printf("done processing error\n");
	}
}

void DTCPConnection::dsh_connectionClosed()
{
	reset();
	connectionClosed();
}

void DTCPConnection::dsh_delayedCloseFinished()
{
	// echo
	delayedCloseFinished();
}

void DTCPConnection::dsh_readyRead()
{
	// echo
	readyRead();
}

void DTCPConnection::dsh_bytesWritten(int x)
{
	// echo
	bytesWritten(x);
}

void DTCPConnection::dsh_error(int)
{
	close();
	error(ErrSocket);
}

void DTCPConnection::dtcp_finished()
{
	JT_DTCP *j = d->j;
	d->j = 0;

	if(j->success()) {
		d->verify = j->verify();
		d->hasVerify = true;

		QString dstr; dstr.sprintf("DTCPConnection: %s [%s] accepted. (verify=%s)\n", d->peer.full().latin1(), d->key.latin1(), d->verify.latin1());
		d->m->client()->debug(dstr);

		if(d->passive) {
			d->ts = QDateTime::currentDateTime();

			// tell the manager to finish verifying it again
			d->m->tryVerify(d->key);
		}
		else {
			d->hostList = j->hostList();
			d->state = Connecting;

			// connect out
			conn();
		}
	}
	else {
		QString dstr; dstr.sprintf("DTCPConnection: %s [%s] refused.\n", d->peer.full().latin1(), d->key.latin1());
		d->m->client()->debug(dstr);

		close();
		error(ErrAsk);
	}
}

void DTCPConnection::reset()
{
	delete d->j;
	d->j = 0;
	delete d->sock;
	d->sock = 0;

	d->state = Idle;
	d->hasVerify = false;
	d->m->unlink(this);
}

void DTCPConnection::onError()
{
	reset();
	error(ErrEstablish);
}

bool DTCPConnection::hasVerify() const
{
	return d->hasVerify;
}


//----------------------------------------------------------------------------
// DTCPManager
//----------------------------------------------------------------------------
class DTCPManager::Private
{
public:
	Private() {}

	ServSock *serv;         
	Client *client;
	DTCPConnectionList conns;
	JT_PushDTCP *pdtcp;
	QStringList hostList;
	QPtrList<DTCPSocketHandler> dshList;
};

DTCPManager::DTCPManager(Client *parent)
:QObject(parent)
{
	d = new Private;
	d->serv = 0;
	d->client = parent;

	d->pdtcp = new JT_PushDTCP(d->client->rootTask());
	connect(d->pdtcp, SIGNAL(incoming(const Jid &, const QString &, const QString &, const QDomElement &)), SLOT(pdtcp_incoming(const Jid &, const QString &, const QString &, const QDomElement &)));
	connect(d->pdtcp, SIGNAL(incomingReverse(const Jid &, const QString &, const QString &, const QStringList &, const QDomElement &)), SLOT(pdtcp_incomingReverse(const Jid &, const QString &, const QString &, const QStringList &, const QDomElement &)));
	connect(d->pdtcp, SIGNAL(error(const Jid &, const QString &, int, const QString &)), SLOT(pdtcp_error(const Jid &, const QString &, int, const QString &)));
}

DTCPManager::~DTCPManager()
{
	delete d->pdtcp;
	delete d->serv;
	delete d;
}

Client *DTCPManager::client() const
{
	return d->client;
}

bool DTCPManager::isActive() const
{
	return (d->serv ? true: false);
}

bool DTCPManager::setServer(int port)
{
	delete d->serv;
	d->serv = 0;

	if(port == -1)
		return false;

	d->serv = new ServSock(port);
	if(!d->serv->ok()) {
		delete d->serv;
		d->serv = 0;
		return false;
	}
	connect(d->serv, SIGNAL(connectionReady(int)), SLOT(connectionReady(int)));

	return true;
}

void DTCPManager::setHostList(const QStringList &list)
{
	d->hostList = list;
}

QStringList DTCPManager::hostList() const
{
	return d->hostList;
}

int DTCPManager::port() const
{
	if(d->serv)
		return d->serv->port();
	else
		return -1;
}

const DTCPConnectionList *DTCPManager::connectionList() const
{
	return &d->conns;
}

void DTCPManager::connectionReady(int s)
{
	DTCPSocketHandler *h = new DTCPSocketHandler(this);
	connect(h, SIGNAL(destroyed()), SLOT(dsh_destroyed()));
	connect(h, SIGNAL(result(bool, const QString &)), SLOT(dsh_result(bool, const QString &)));
	d->dshList.append(h);
	h->handle(s);
}

void DTCPManager::pdtcp_incoming(const Jid &from, const QString &id, const QString &key, const QDomElement &comment)
{
	if(!isActive()) {
		d->pdtcp->respondError(from, id, 501, "Not supported");
		return;
	}

	DTCPConnection *c = findRemoteConnection(from, key);
	if(c) {
		d->pdtcp->respondError(from, id, 403, "Key in use");
	}
	else {
		QString verify = genKey();

		// create a "waiting" connection
		DTCPConnection *c = new DTCPConnection(this);
		c->waitForSocket(from, key, verify, comment, id);

		incoming(c);

		// destroy if not accepted
		/*if(c->state() == DTCPConnection::WaitingForSocket) {
			printf("deleting\n");
			delete c;
		}*/
	}
}

void DTCPManager::pdtcp_incomingReverse(const Jid &from, const QString &id, const QString &key, const QStringList &hostList, const QDomElement &comment)
{
	DTCPConnection *c = findRemoteConnection(from, key);
	if(c) {
		d->pdtcp->respondError(from, id, 403, "Key in use");
	}
	else {
		QString verify = genKey();

		DTCPConnection *c = new DTCPConnection(this);
		c->waitForAccept(from, key, verify, hostList, comment, id);

		incoming(c);

		// destroy if not accepted
		/*if(c->state() == DTCPConnection::WaitingForAccept) {
			printf("deleting\n");
			delete c;
		}*/
	}
}

void DTCPManager::pdtcp_error(const Jid &from, const QString &key, int, const QString &)
{
	DTCPConnection *c = findConnection(from, key);
	if(!c)
		return;
	// only respond to errors if we are waiting for a socket
	if(c->wantsIncomingSocket())
		c->onError();
}

void DTCPManager::dsh_result(bool success, const QString &)
{
	// we don't care about failed incoming connections
	if(!success)
		return;

	DTCPSocketHandler *h = (DTCPSocketHandler *)sender();
	DTCPConnection *c = findConnection(h->peer(), h->key());
	if(!c) {
		delete h;
		return;
	}
	if(c->wantsIncomingSocket())
		c->setSocketHandler(h);
}

void DTCPManager::doAccept(DTCPConnection *c, const QString &id)
{
	// respond with our list of hosts
	QStringList hl;
	for(QStringList::ConstIterator it = d->hostList.begin(); it != d->hostList.end(); ++it)
		hl += (*it) + ':' + QString::number(d->serv->port());
	d->pdtcp->respondSuccess(c->peer(), id, hl, c->verify());
}

void DTCPManager::doAcceptReverse(DTCPConnection *c, const QString &id)
{
	d->pdtcp->respondReverseSuccess(c->peer(), id, c->verify());
}

void DTCPManager::doReject(DTCPConnection *c, const QString &id)
{
	d->pdtcp->respondError(c->peer(), id, 501, "Rejected");
}

DTCPConnection *DTCPManager::findConnection(const Jid &peer, const QString &key) const
{
	DTCPConnectionListIt it(d->conns);
	for(DTCPConnection *c; (c = it.current()); ++it) {
		printf("finding: %s,%s\n", c->peer().full().latin1(), c->key().latin1());
		if(c->peer().compare(peer) && c->key() == key)
			return c;
	}
	return 0;
}

DTCPConnection *DTCPManager::findLocalConnection(const Jid &peer, const QString &key) const
{
	DTCPConnectionListIt it(d->conns);
	for(DTCPConnection *c; (c = it.current()); ++it) {
		printf("finding: %s,%s\n", c->peer().full().latin1(), c->key().latin1());
		if(c->peer().compare(peer) && c->key() == key && !c->isRemote())
			return c;
	}
	return 0;
}

DTCPConnection *DTCPManager::findRemoteConnection(const Jid &peer, const QString &key) const
{
	DTCPConnectionListIt it(d->conns);
	for(DTCPConnection *c; (c = it.current()); ++it) {
		printf("finding: %s,%s\n", c->peer().full().latin1(), c->key().latin1());
		if(c->peer().compare(peer) && c->key() == key && c->isRemote())
			return c;
	}
	return 0;
}

DTCPConnection *DTCPManager::findWaitingConnection(const QString &key) const
{
	DTCPConnectionListIt it(d->conns);
	for(DTCPConnection *c; (c = it.current()); ++it) {
		printf("finding: %s,%s,%d\n", c->peer().full().latin1(), c->key().latin1(), c->wantsIncomingSocket());
		if(c->key() == key && c->wantsIncomingSocket())
			return c;
	}
	return 0;
}

QString DTCPManager::genKey() const
{
	QString key;

	for(int i = 0; i < 4; ++i) {
		int word = rand() & 0xffff;
		for(int n = 0; n < 4; ++n) {
			QString s;
			s.sprintf("%x", (word >> (n * 4)) & 0xf);
			key.append(s);
		}
	}

	return key;
}

QString DTCPManager::genUniqueKey(const Jid &j) const
{
	// get unused key
	QString key;
	while(1) {
		key = genKey();
		if(!findConnection(j, key))
			break;
	}

	return key;
}

void DTCPManager::link(DTCPConnection *c)
{
	d->conns.append(c);
}

void DTCPManager::unlink(DTCPConnection *c)
{
	d->conns.remove(c);
}

void DTCPManager::tryVerify(const QString &key)
{
	QPtrListIterator<DTCPSocketHandler> it(d->dshList);
	for(DTCPSocketHandler *dsh; (dsh = it.current()); ++it) {
		if(dsh->waitingOn() && dsh->key() == key)
			dsh->tryVerify();
	}
}

void DTCPManager::dsh_destroyed()
{
	DTCPSocketHandler *dsh = (DTCPSocketHandler *)sender();

	d->dshList.removeRef(dsh);
}


//----------------------------------------------------------------------------
// JT_DTCP
//----------------------------------------------------------------------------
JT_DTCP::JT_DTCP(Task *parent)
:Task(parent)
{
}

JT_DTCP::~JT_DTCP()
{
}

void JT_DTCP::request(const Jid &_to, const QString &key, const QDomElement &comment)
{
	mode = 0;
	to = _to;
	v_key = key;
	iq = createIQ(doc(), "set", to.full(), id());
	QDomElement query = doc()->createElement("query");
	query.setAttribute("xmlns", "jabber:iq:dtcp");
	iq.appendChild(query);
	query.appendChild(textTag(doc(), "key", v_key));
	query.appendChild(comment);
}

void JT_DTCP::requestReverse(const Jid &_to, const QString &key, const QStringList &hostList, const QDomElement &comment)
{
	mode = 1;
	to = _to;
	v_key = key;
	iq = createIQ(doc(), "set", to.full(), id());
	QDomElement query = doc()->createElement("query");
	query.setAttribute("xmlns", "jabber:iq:dtcp");
	iq.appendChild(query);
	for(QStringList::ConstIterator it = hostList.begin(); it != hostList.end(); ++it) {
		QString host, port;
		int n = (*it).find(':');
		if(n == -1) {
			host = *it;
			port = "";
		}
		else {
			host = (*it).mid(0, n);
			port = (*it).mid(n+1);
		}
		QDomElement h = textTag(doc(), "host", host);
		if(!port.isEmpty())
			h.setAttribute("port", port);
		query.appendChild(h);
	}
	query.appendChild(textTag(doc(), "key", v_key));
	query.appendChild(comment);
}

void JT_DTCP::onGo()
{
	send(iq);
}

bool JT_DTCP::take(const QDomElement &x)
{
	Jid from(x.attribute("from"));
	if(x.attribute("id") != id() || !to.compare(from))
		return false;

	if(x.attribute("type") == "result") {
		if(mode == 0) {
			QDomElement q = queryTag(x);
			v_hostList.clear();
			for(QDomNode n = q.firstChild(); !n.isNull(); n = n.nextSibling()) {
				QDomElement i = n.toElement();
				if(i.isNull())
					continue;
				if(i.tagName() == "host" && i.hasAttribute("port")) {
					QString str = tagContent(i) + ':' + i.attribute("port");
					v_hostList += str;
				}
			}

			bool found;
			QDomElement v = findSubTag(q, "verify", &found);
			if(found) {
				v_verify = tagContent(v);
			}

			setSuccess();
		}
		else {
			QDomElement q = queryTag(x);
			bool found;
			QDomElement v = findSubTag(q, "verify", &found);
			if(found) {
				v_verify = tagContent(v);
			}

			setSuccess();
		}
	}
	else {
		setError(x);
	}

	return true;
}

QStringList JT_DTCP::hostList() const
{
	return v_hostList;
}

QString JT_DTCP::key() const
{
	return v_key;
}

QString JT_DTCP::verify() const
{
	return v_verify;
}

Jid JT_DTCP::jid() const
{
	return to;
}


//----------------------------------------------------------------------------
// JT_PushDTCP
//----------------------------------------------------------------------------
JT_PushDTCP::JT_PushDTCP(Task *parent)
:Task(parent)
{
}

JT_PushDTCP::~JT_PushDTCP()
{
}

bool JT_PushDTCP::take(const QDomElement &e)
{
	// must be an iq-set tag
	if(e.tagName() != "iq")
		return false;

	bool ok = false;
	if(e.attribute("type") == "set")
		ok = true;
	else if(e.attribute("type") == "error" && !e.hasAttribute("id"))
		ok = true;
	else if(e.attribute("type") == "result" && !e.hasAttribute("id"))
		ok = true;
	if(!ok)
		return false;
	if(queryNS(e) != "jabber:iq:dtcp")
		return false;

	Jid from(e.attribute("from"));
	QDomElement q = queryTag(e);
	bool found;
	QDomElement k = findSubTag(q, "key", &found);
	if(!found)
		return true;
	QString key = tagContent(k);

	if(e.attribute("type") == "set") {
		QStringList hostList;
		hostList.clear();
		for(QDomNode n = q.firstChild(); !n.isNull(); n = n.nextSibling()) {
			QDomElement i = n.toElement();
			if(i.isNull())
				continue;
			if(i.tagName() == "host" && i.hasAttribute("port")) {
				QString str = tagContent(i) + ':' + i.attribute("port");
				hostList += str;
			}
		}
		QDomElement comment = findSubTag(q, "comment", &found);

		if(hostList.isEmpty())
			incoming(from, e.attribute("id"), key, comment);
		else
			incomingReverse(from, e.attribute("id"), key, hostList, comment);
	}
	else {
		QString str = "";
		int code = 0;
		QDomElement err = findSubTag(e, "error", &found);
		if(found) {
			str = tagContent(err);
			code = err.attribute("code").toInt();
		}
		error(from, key, code, str);
	}

	return true;
}

void JT_PushDTCP::respondSuccess(const Jid &to, const QString &id, const QStringList &hostList, const QString &verify)
{
	QDomElement iq = createIQ(doc(), "result", to.full(), id);
	QDomElement query = doc()->createElement("query");
	query.setAttribute("xmlns", "jabber:iq:dtcp");
	iq.appendChild(query);
	query.appendChild(textTag(doc(), "verify", verify));
	for(QStringList::ConstIterator it = hostList.begin(); it != hostList.end(); ++it) {
		QString host, port;
		int n = (*it).find(':');
		if(n == -1) {
			host = *it;
			port = "";
		}
		else {
			host = (*it).mid(0, n);
			port = (*it).mid(n+1);
		}
		QDomElement h = textTag(doc(), "host", host);
		if(!port.isEmpty())
			h.setAttribute("port", port);
		query.appendChild(h);
	}

	send(iq);
}

void JT_PushDTCP::respondReverseSuccess(const Jid &to, const QString &id, const QString &verify)
{
	QDomElement iq = createIQ(doc(), "result", to.full(), id);
	QDomElement query = doc()->createElement("query");
	query.setAttribute("xmlns", "jabber:iq:dtcp");
	iq.appendChild(query);
	query.appendChild(textTag(doc(), "verify", verify));

	send(iq);
}

void JT_PushDTCP::respondError(const Jid &to, const QString &id, int code, const QString &str)
{
	QDomElement iq = createIQ(doc(), "error", to.full(), id);
	QDomElement err = textTag(doc(), "error", str);
	err.setAttribute("code", QString::number(code));
	iq.appendChild(err);

	send(iq);
}


//----------------------------------------------------------------------------
// ServSocket
//----------------------------------------------------------------------------
ServSock::ServSock(int port)
:QServerSocket(port, 16)
{
}

void ServSock::newConnection(int x)
{
	connectionReady(x);
}
