/*
 * psicon.cpp - core of Psi
 * Copyright (C) 2001, 2002  Justin Karneges
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include"psicon.h"

#include<qptrlist.h>
#include<qapplication.h>
#include<qdesktopwidget.h>
#include<qguardedptr.h>

#include"psiaccount.h"
#include"contactview.h"
#include"common.h"
#include"mainwin.h"
#include"idle.h"
#include"accountdlg.h"
#include"statusdlg.h"
#include"optionsdlg.h"
#include"groupchatdlg.h"
#include"userlist.h"
#include"eventdlg.h"
#include"eventdb.h"


//----------------------------------------------------------------------------
// AccountsComboBox
//----------------------------------------------------------------------------
AccountsComboBox::AccountsComboBox(PsiCon *_psi, QWidget *parent)
:QComboBox(parent)
{
	psi = _psi;
	connect(psi, SIGNAL(accountCountChanged()), this, SLOT(updateAccounts()));
	connect(psi, SIGNAL(destroyed()), SLOT(deleteMe()));
	connect(this, SIGNAL(activated(int)), this, SLOT(changeAccount()));
	pa = 0;
	setSizePolicy(QSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed ));

	if(!psi->accountList().isEmpty())
		setAccount(psi->accountList().getFirst());
}

AccountsComboBox::~AccountsComboBox()
{
}

void AccountsComboBox::setAccount(PsiAccount *_pa)
{
	pa = _pa;
	updateAccounts();
}

void AccountsComboBox::changeAccount()
{
	int i = currentItem();

	PsiAccountListIt it(psi->accountList());
	int n = 0;
	bool found = false;
	for(PsiAccount *p; (p = it.current()); ++it) {
		if(i == n) {
			pa = p;
			found = true;
			break;
		}
		++n;
	}
	if(!found)
		pa = 0;

	activated(pa);
}

void AccountsComboBox::updateAccounts()
{
	clear();
	PsiAccountListIt it(psi->accountList());
	int n = 0;
	bool found = false;
	for(PsiAccount *p; (p = it.current()); ++it) {
		insertItem(p->nameWithJid());
		if(p == pa) {
			setCurrentItem(n);
			found = true;
		}
		++n;
	}
	if(!found) {
		// choose a different account
		if(psi->accountList().isEmpty())
			pa = 0;
		else {
			pa = psi->accountList().getFirst();
			setCurrentItem(0);
		}

		activated(pa);
	}
}

void AccountsComboBox::deleteMe()
{
	delete this;
}


//----------------------------------------------------------------------------
// PsiCon
//----------------------------------------------------------------------------
struct item_dialog
{
	QWidget *widget;
	QString className;
};

class PsiCon::Private
{
public:
	Private() {}

	QPtrList<PsiAccount> list;
	UserProfile pro;
	QString lastStatusString;
	bool useSound;
	MainWin *mainwin;
	Idle idle;
	QPtrList<item_dialog> dialogList;
	int eventId;
	QStringList recentGCList, recentBrowseList;
	EDB *edb;
};

PsiCon::PsiCon()
:QObject(0)
{
	//pdb(DEBUG_JABCON, QString("%1 v%2\n By Justin Karneges\n    infiniti@affinix.com\n\n").arg(PROG_NAME).arg(PROG_VERSION));

	d = new Private;

	d->lastStatusString = "";
	useSound = true;
	d->mainwin = 0;

	d->dialogList.setAutoDelete(true);

	d->eventId = 0;
	d->edb = new EDBFlatFile;
}

PsiCon::~PsiCon()
{
	deinit();

	delete d->edb;
	delete d;
}

bool PsiCon::init()
{
	// load the profile
	d->pro.reset();
	d->pro.fromFile(pathToProfileConfig(activeProfile));

	d->recentGCList = d->pro.recentGCList;
	d->lastStatusString = d->pro.lastStatusString;
	useSound = d->pro.useSound;

	option = d->pro.prefs;

	// first thing, try to load the iconset
	if(!is.load(option.iconset)) {
		option.iconset = "stellar";
		if(!is.load(option.iconset)) {
			QMessageBox::critical(0, tr("Error"), tr("Unable to load iconset!  Please make sure Psi is properly installed."));
			return false;
		}
	}

	// setup the main window
	d->mainwin = new MainWin(option.alwaysOnTop, (option.useDock && option.dockToolMW), this, "psimain");
	d->mainwin->setUseDock(option.useDock);
	d->mainwin->setUsingSSL(false);

	connect(d->mainwin, SIGNAL(closeProgram()), SLOT(closeProgram()));
	connect(d->mainwin, SIGNAL(changeProfile()), SLOT(changeProfile()));
	connect(d->mainwin, SIGNAL(doManageAccounts()), SLOT(doManageAccounts()));
	connect(d->mainwin, SIGNAL(doGroupChat()), SLOT(doGroupChat()));
	connect(d->mainwin, SIGNAL(blankMessage()), SLOT(doNewBlankMessage()));
	connect(d->mainwin, SIGNAL(statusChanged(int)), SLOT(statusMenuChanged(int)));
	connect(d->mainwin, SIGNAL(doOptions()), SLOT(doOptions()));
	connect(d->mainwin, SIGNAL(recvNextEvent()), SLOT(recvNextEvent()));
	connect(this, SIGNAL(emitOptionsUpdate()), d->mainwin, SLOT(optionsUpdate()));

	connect(this, SIGNAL(emitOptionsUpdate()), d->mainwin->cvlist, SLOT(optionsUpdate()));

	// if the coordinates are out of the desktop bounds, reset to the top left
	QRect mwgeom = d->pro.mwgeom;
	QDesktopWidget *pdesktop = QApplication::desktop();
	int nscreen = pdesktop->screenNumber(mwgeom.topLeft());
	QRect r = pdesktop->screenGeometry(nscreen);
	if(mwgeom.x() >= r.right() || mwgeom.y() >= r.bottom())
		mwgeom.moveBy(-mwgeom.left()+20+r.left(), -mwgeom.top()+20+r.top());
	d->mainwin->move(mwgeom.x(), mwgeom.y());
	d->mainwin->resize(mwgeom.width(), mwgeom.height());

	if(!(option.useDock && option.dockHideMW))
		d->mainwin->show();

	d->idle.start();

	// load accounts
	loadAccounts(d->pro.acc);

	return true;
}

void PsiCon::deinit()
{
	// this deletes all dialogs except for mainwin
	deleteAllDialogs();

	d->idle.stop();

	// shut down all accounts
	UserAccountList acc = unloadAccounts();

	// shut down mainwin
	QRect mwgeom;
	mwgeom.setX(d->mainwin->x());
	mwgeom.setY(d->mainwin->y());
	mwgeom.setWidth(d->mainwin->width());
	mwgeom.setHeight(d->mainwin->height());
	delete d->mainwin;
	d->mainwin = 0;

	// unload iconset
	is.unload();

	// save profile
	d->pro.mwgeom = mwgeom;
	d->pro.recentGCList = d->recentGCList;
	d->pro.lastStatusString = d->lastStatusString;
	d->pro.useSound = useSound;

	d->pro.acc = acc;
	d->pro.prefs = option;
	d->pro.toFile(pathToProfileConfig(activeProfile));
}

PsiAccount *PsiCon::loadAccount(const UserAccount &acc)
{
	PsiAccount *pa = new PsiAccount(acc, this);
	connect(&d->idle, SIGNAL(secondsIdle(int)), pa, SLOT(secondsIdle(int)));
	connect(pa, SIGNAL(updatedActivity()), SLOT(pa_updatedActivity()));
	connect(pa, SIGNAL(updatedAccount()), SLOT(pa_updatedAccount()));
	connect(pa, SIGNAL(queueChanged()), SLOT(queueChanged()));
	accountAdded(pa);
	return pa;
}

void PsiCon::loadAccounts(const UserAccountList &list)
{
	if(list.count() > 0) {
		for(UserAccountList::ConstIterator it = list.begin(); it != list.end(); ++it)
			loadAccount(*it);
	}
	else {
		// if there are no accounts, then prompt the user to add one
		AccountAddDlg *w = new AccountAddDlg(this, 0);
		w->show();
	}
}

UserAccountList PsiCon::unloadAccounts()
{
	UserAccountList acc;

	QPtrListIterator<PsiAccount> it(d->list);
	for(PsiAccount *pa; (pa = it.current());) {
		acc += pa->userAccount();
		delete pa;
	}

	return acc;
}

const PsiAccountList & PsiCon::accountList() const
{
	return d->list;
}

ContactView *PsiCon::contactView() const
{
	if(d->mainwin)
		return d->mainwin->cvlist;
	else
		return 0;
}

EDB *PsiCon::edb() const
{
	return d->edb;
}

void PsiCon::link(PsiAccount *pa)
{
	d->list.append(pa);
	accountCountChanged();
}

void PsiCon::unlink(PsiAccount *pa)
{
	d->list.remove(pa);
	accountCountChanged();
}

void PsiCon::closeProgram()
{
	quit(QuitProgram);
}

void PsiCon::changeProfile()
{
	bool ok = true;
	PsiAccountListIt it(d->list);
	for(PsiAccount *pa; (pa = it.current()); ++it) {
		if(pa->isActive()) {
			ok = false;
			break;
		}
	}

	if(!ok) {
		QMessageBox::information(0, CAP(tr("Error")), tr("Please disconnect before changing the profile."));
		return;
	}

	quit(QuitProfile);
}

void PsiCon::doManageAccounts()
{
	AccountManageDlg *w = (AccountManageDlg *)dialogFind("AccountManageDlg");
	if(w)
		bringToFront(w);
	else {
		w = new AccountManageDlg(this);
		w->show();
	}
}

void PsiCon::doGroupChat()
{
	PsiAccount *pa = d->list.getFirst();
	if(!pa)
		return;

	GCJoinDlg *w = new GCJoinDlg(this, pa);
	w->show();
}

void PsiCon::doNewBlankMessage()
{
	PsiAccount *pa = d->list.getFirst();
	if(!pa)
		return;

	EventDlg *w = createEventDlg("", pa);
	w->show();
}

EventDlg *PsiCon::createEventDlg(const QString &to, PsiAccount *pa)
{
	EventDlg *w = new EventDlg(to, this, pa);
	connect(this, SIGNAL(emitOptionsUpdate()), w, SLOT(optionsUpdate()));
	return w;
}

void PsiCon::updateContactGlobal(PsiAccount *pa, const Jid &j)
{
	QPtrListIterator<item_dialog> it(d->dialogList);
	for(item_dialog *i; (i = it.current()); ++it) {
		if(i->className == "EventDlg") {
			EventDlg *e = (EventDlg *)i->widget;
			if(e->psiAccount() == pa)
				e->updateContact(j);
		}
	}
}

QWidget *PsiCon::dialogFind(const char *className)
{
	QPtrListIterator<item_dialog> it(d->dialogList);
	for(item_dialog *i; (i = it.current()); ++it) {
		// does the classname and jid match?
		if(i->className == className) {
			return i->widget;
		}
	}
	return 0;
}

void PsiCon::dialogRegister(QWidget *w)
{
	item_dialog *i = new item_dialog;
	i->widget = w;
	i->className = w->className();
	d->dialogList.append(i);
}

void PsiCon::dialogUnregister(QWidget *w)
{
	QPtrListIterator<item_dialog> it(d->dialogList);
	for(item_dialog *i; (i = it.current()); ++it) {
		if(i->widget == w) {
			d->dialogList.removeRef(i);
			return;
		}
	}
}

void PsiCon::deleteAllDialogs()
{
	QPtrListIterator<item_dialog> it(d->dialogList);
	for(item_dialog *i; (i = it.current());)
		delete i->widget;
	d->dialogList.clear();
}

AccountsComboBox *PsiCon::accountsComboBox(QWidget *parent)
{
	AccountsComboBox *acb = new AccountsComboBox(this, parent);
	return acb;
}

bool PsiCon::isValid(PsiAccount *pa)
{
	if(d->list.findRef(pa) == -1)
		return false;
	else
		return true;
}

void PsiCon::createAccount(const QString &name, const QString &host, int port, bool ssl, const QString &user, const QString &pass)
{
	UserAccount acc;
	acc.name = name;
	acc.host = host;
	acc.port = port;
	acc.opt_ssl = ssl;
	acc.user = user;
	if(!pass.isEmpty()) {
		acc.opt_pass = true;
		acc.pass = pass;
	}
	acc.vhost = acc.host;

	PsiAccount *pa = loadAccount(acc);
	saveAccounts();

	// pop up the modify dialog so the user can customize the new account
	pa->modify();
}

void PsiCon::modifyAccount(PsiAccount *pa)
{
	pa->modify();
}

void PsiCon::removeAccount(PsiAccount *pa)
{
	accountRemoved(pa);
	delete pa;
}

void PsiCon::statusMenuChanged(int x)
{
	if(x == STATUS_OFFLINE) {
		setGlobalStatus(Status("","",0,false));
	}
	else {
		if(x == STATUS_ONLINE && !option.askOnline) {
			setGlobalStatus(Status());
		}
		else {
			QString str;
			if(d->lastStatusString.isEmpty())
				str = tr("I am away from my computer.  Please leave a message.");
			else
				str = d->lastStatusString; // default to old away message

			StatusSetDlg *w = new StatusSetDlg(this, makeStatus(x, str));
			connect(w, SIGNAL(set(const Status &)), SLOT(setStatusFromDialog(const Status &)));
			w->show();
		}
	}
}

void PsiCon::setStatusFromDialog(const Status &s)
{
	d->lastStatusString = s.status();
	setGlobalStatus(s);
}

void PsiCon::setGlobalStatus(const Status &s)
{
	// globally set each account
	PsiAccountListIt it(d->list);
	for(PsiAccount *pa; (pa = it.current()); ++it)
		pa->setStatus(s);
}

void PsiCon::pa_updatedActivity()
{
	PsiAccount *pa = (PsiAccount *)sender();
	accountUpdated(pa);

	updateMainwinStatus();
}

void PsiCon::pa_updatedAccount()
{
	PsiAccount *pa = (PsiAccount *)sender();
	accountUpdated(pa);

	saveAccounts();
}

void PsiCon::saveAccounts()
{
	UserAccountList acc;
	PsiAccountListIt it(d->list);
	for(PsiAccount *pa; (pa = it.current()); ++it)
		acc += pa->userAccount();

	d->pro.acc = acc;
	d->pro.toFile(pathToProfileConfig(activeProfile));
}

void PsiCon::updateMainwinStatus()
{
	bool active = false;
	bool loggedIn = false;
	int state = STATUS_ONLINE;
	PsiAccountListIt it(d->list);
	for(PsiAccount *pa; (pa = it.current()); ++it) {
		if(pa->isActive())
			active = true;
		if(pa->loggedIn()) {
			loggedIn = true;
			state = makeSTATUS(pa->status());
		}
	}
	if(loggedIn)
		d->mainwin->decorateButton(state);
	else {
		if(active)
			d->mainwin->decorateButton(-1);
		else
			d->mainwin->decorateButton(STATUS_OFFLINE);
	}
}

void PsiCon::setToggles(bool tog_offline, bool tog_away, bool tog_agents)
{
	if(d->list.count() > 1)
		return;

	d->mainwin->cvlist->setShowOffline(tog_offline);
	d->mainwin->cvlist->setShowAway(tog_away);
	d->mainwin->cvlist->setShowAgents(tog_agents);
}

void PsiCon::getToggles(bool *tog_offline, bool *tog_away, bool *tog_agents)
{
	*tog_offline = d->mainwin->cvlist->isShowOffline();
	*tog_away = d->mainwin->cvlist->isShowAway();
	*tog_agents = d->mainwin->cvlist->isShowAgents();
}

void PsiCon::doOptions()
{
	OptionsDlg *w = (OptionsDlg *)dialogFind("OptionsDlg");
	if(w)
		bringToFront(w);
	else {
		w = new OptionsDlg(this, option);
		connect(w, SIGNAL(applyOptions(const Options &)), SLOT(slotApplyOptions(const Options &)));
		w->show();
	}
}

void PsiCon::slotApplyOptions(const Options &opt)
{
	QString oldiconset = option.iconset;
	option = opt;

	// change icon set
	if(option.iconset != oldiconset) {
		is.unload();
		if(!is.load(option.iconset)) {
			QMessageBox::critical(0, tr("Error"), QString(tr("Unable to load the \"%1\" iconset.")).arg(option.iconset));
			option.iconset = oldiconset;
			is.load(option.iconset);
		}
	}

	d->mainwin->setWindowOpts(option.alwaysOnTop, (option.useDock && option.dockToolMW));
	d->mainwin->setUseDock(option.useDock);

	emitOptionsUpdate();

	// save just the options
	d->pro.prefs = option;
	d->pro.toFile(pathToProfileConfig(activeProfile));
}

int PsiCon::getId()
{
	return d->eventId++;
}

int PsiCon::queueCount()
{
	int total = 0;
	PsiAccountListIt it(d->list);
	for(PsiAccount *pa; (pa = it.current()); ++it)
		total += pa->eventQueue()->count();
	return total;
}

PsiAccount *PsiCon::queueLowestEventId()
{
	PsiAccount *low = 0;
	int low_id = 0;

	PsiAccountListIt it(d->list);
	for(PsiAccount *pa; (pa = it.current()); ++it) {
		int n = pa->eventQueue()->nextId();
		if(n == -1)
			continue;
		if(!low || n < low_id) {
			low = pa;
			low_id = n;
		}
	}

	return low;
}

void PsiCon::queueChanged()
{
	Anim *nextAnim = 0;
	int nextAmount = queueCount();
	PsiAccount *pa = queueLowestEventId();
	if(pa)
		nextAnim = &is.event2anim(pa->eventQueue()->peekNext());

	d->mainwin->updateReadNext(nextAnim, nextAmount);
}

void PsiCon::recvNextEvent()
{
	PsiAccount *pa = queueLowestEventId();
	if(pa)
		pa->openNextEvent();
}

void PsiCon::playSound(const QString &str)
{
	if(str.isEmpty() || !useSound)
		return;

	soundPlay(str);
}

void PsiCon::raiseMainwin()
{
	d->mainwin->showNoFocus();
}

const QStringList & PsiCon::recentGCList() const
{
	return d->recentGCList;
}

void PsiCon::recentGCAdd(const QString &str)
{
	// remove it if we have it
	for(QStringList::Iterator it = d->recentGCList.begin(); it != d->recentGCList.end(); ++it) {
		if(*it == str) {
			d->recentGCList.remove(it);
			break;
		}
	}

	// put it in the front
	d->recentGCList.prepend(str);

	// trim the list if bigger than 10
	while(d->recentGCList.count() > 10)
		d->recentGCList.remove(d->recentGCList.fromLast());
}

const QStringList & PsiCon::recentBrowseList() const
{
	return d->recentBrowseList;
}

void PsiCon::recentBrowseAdd(const QString &str)
{
	// remove it if we have it
	for(QStringList::Iterator it = d->recentBrowseList.begin(); it != d->recentBrowseList.end(); ++it) {
		if(*it == str) {
			d->recentBrowseList.remove(it);
			break;
		}
	}

	// put it in the front
	d->recentBrowseList.prepend(str);

	// trim the list if bigger than 10
	while(d->recentBrowseList.count() > 10)
		d->recentBrowseList.remove(d->recentBrowseList.fromLast());
}


//----------------------------------------------------------------------------
// EventQueue
//----------------------------------------------------------------------------
EventQueue::EventQueue(PsiCon *_psi)
{
	psi = _psi;
	list.setAutoDelete(true);
}

EventQueue::~EventQueue()
{
}

int EventQueue::nextId() const
{
	QPtrListIterator<EventItem> it(list);
	EventItem *i = it.current();
	if(!i)
		return -1;
	return i->id();
}

int EventQueue::count() const
{
	return list.count();
}

int EventQueue::count(const Jid &j, bool compareRes) const
{
	int total = 0;

	QPtrListIterator<EventItem> it(list);
	for(EventItem *i; (i = it.current()); ++it) {
		Jid j2(i->event()->jid());
		if(j.compare(j2, compareRes))
			++total;
	}

	return total;
}

void EventQueue::enqueue(PsiEvent *e)
{
	EventItem *i = new EventItem(e, psi->getId());
	list.append(i);
}

PsiEvent *EventQueue::dequeue(const Jid &j, bool compareRes)
{
	QPtrListIterator<EventItem> it(list);
	for(EventItem *i; (i = it.current()); ++it) {
		PsiEvent *e = i->event();
		Jid j2(e->jid());
		if(j.compare(j2, compareRes)) {
			list.removeRef(i);
			return e;
		}
	}

	return 0;
}

PsiEvent *EventQueue::peek(const Jid &j, bool compareRes) const
{
	QPtrListIterator<EventItem> it(list);
	for(EventItem *i; (i = it.current()); ++it) {
		PsiEvent *e = i->event();
		Jid j2(e->jid());
		if(j.compare(j2, compareRes)) {
			return e;
		}
	}

	return 0;
}

PsiEvent *EventQueue::dequeueNext()
{
	QPtrListIterator<EventItem> it(list);
	EventItem *i = it.current();
	if(!i)
		return 0;
	PsiEvent *e = i->event();
	list.remove(it);
	return e;
}

PsiEvent *EventQueue::peekNext() const
{
	QPtrListIterator<EventItem> it(list);
	EventItem *i = it.current();
	if(!i)
		return 0;
	return i->event();
}

bool EventQueue::hasChats(const Jid &j, bool compareRes) const
{
	QPtrListIterator<EventItem> it(list);
	for(EventItem *i; (i = it.current()); ++it) {
		PsiEvent *e = i->event();
		if(e->type() == PsiEvent::Message) {
			MessageEvent *me = (MessageEvent *)e;
			if(j.compare(me->from(), compareRes) && me->message().type() == "chat")
				return true;
		}
	}

	return false;
}

// this function extracts all chats from the queue, and returns a list of queue positions
void EventQueue::extractChats(QPtrList<PsiEvent> *el, const Jid &j, bool compareRes)
{
	QPtrListIterator<EventItem> it(list);
	for(EventItem *i; (i = it.current());) {
		PsiEvent *e = i->event();
		if(e->type() == PsiEvent::Message) {
			MessageEvent *me = (MessageEvent *)e;
			if(j.compare(me->from(), compareRes) && me->message().type() == "chat") {
				el->append(me);
				list.remove(it);
				continue;
			}
		}
		++it;
	}
}

void EventQueue::clear()
{
	list.clear();
}

// this function removes all events associated with the input jid
void EventQueue::clear(const Jid &j, bool compareRes)
{
	QPtrListIterator<EventItem> it(list);
	for(EventItem *i; (i = it.current());) {
		PsiEvent *e = i->event();
		Jid j2(e->jid());
		if(j.compare(j2, compareRes))
			list.removeRef(i);
		else
			++it;
	}
}
